/****************************************************************************
**
** Copyright (C) 2006-2009 fullmetalcoder <fullmetalcoder@hotmail.fr>
**
** This file is part of the Edyuk project <http://edyuk.org>
** 
** This file may be used under the terms of the GNU General Public License
** version 3 as published by the Free Software Foundation and appearing in the
** file GPL.txt included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef _QDOCUMENT_LINE_P_H_
#define _QDOCUMENT_LINE_P_H_

#include "qce-config.h"

/*!
	\file qdocumentline_p.h
	\brief Definition of QDocumentLineHandle
*/

#include "qformat.h"

#include "qdocumentline.h"

#include <QPair>
#include <QList>
#include <QString>
#include <QVector>

#include <QTextLayout>

class QDocument;
class QDocumentLine;
class QDocumentBuffer;
class QDocumentPrivate;

class QCE_EXPORT QDocumentLineHandle
{
	friend class QDocument;
	friend class QDocumentLine;
	friend class QDocumentBuffer;
	friend class QDocumentPrivate;
	
	public:
		QDocumentLineHandle();
		QDocumentLineHandle(const QString& s);
		QDocumentLineHandle(const QChar *d, int size);
		
		int count() const;
		int length() const;
		
		QString text() const;
		
		int nextNonSpaceChar(uint pos) const;
		int previousNonSpaceChar(int pos) const;
		
		bool hasFlag(int flag) const;
		void setFlag(int flag, bool y = true);
		
		inline const QVector<int>& tokens() const { return m_tokens; }
		
		void clearTokens();
		void addToken(int index, int length, int tokenId);
		void setTokens(const QVector<int>& tokens);
		
		inline const QVector<int>& contexts() const { return m_contexts; }
		
		void clearContexts();
		void addContext(int index, int length, int contextId);
		void setContexts(const QVector<int>& contexts);
		
		void clearOverlays();
		void addOverlay(const QFormatRange& over);
		void removeOverlay(const QFormatRange& over);
		
		void shiftOverlays(int position, int offset);
		
		inline QString& textBuffer() { setFlag(QDocumentLine::TextChanged, true); return m_text; }
		
		~QDocumentLineHandle();
		
	private:
		void layout();
		void applyOverlays();
		
		void compose();
		QList<QTextLayout::FormatRange> decorations() const;
		
		// ideally
		quint8 m_state;
		QString m_text;
		
		// tokencount:int tokendata:{index:int, length:int, tokenid:int}+ overlaydata:{index:int, length:int, formatid:int}+
		QVector<int> m_tokens;
		
		// {index, length, id}+ index=-1 means continuing from prev, length=-1 means continuing beyond
		QVector<int> m_contexts;
		
		union {
			QTextLayout *layout;
			QVector<int> *data;
		} m_render;
};

Q_DECLARE_TYPEINFO(QDocumentLineHandle*, Q_PRIMITIVE_TYPE);

#endif // !_QDOCUMENT_LINE_P_H_
